#!/usr/bin/python
# -*- coding: utf-8 -*- 

#  MIT License
#
#  Copyright (c) 2019 shoucf
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

from utils import *


def CheckHadoop(args):
    path = args.confFolder
    if not os.path.exists(path):
        Log.log_error("'%s' is not a dir!" % path)
        sys.exit(0)
    abs_path = os.path.abspath(path)
    import HadoopCheck
    HadoopCheck.run(abs_path)


def CheckSpark(args):
    path = args.confFolder
    if not os.path.exists(path):
        Log.log_error("'%s' is not a dir!" % path)
        sys.exit(0)
    abs_path = os.path.abspath(path)
    import SparkCheck
    SparkCheck.run(abs_path)


def CheckMySQL(args):
    un = args.username
    pw = args.password
    if args.host:
        host = args.host
    else:
        host = "127.0.0.1"
    if args.port:
        port = args.port
    else:
        port = 3306
    import MySQLCheck
    MySQLCheck.run(un, pw, host=host, port=port)


def CheckRedis(args):
    path = args.confFolder
    if not os.path.exists(path):
        Log.log_error("'%s' is not a dir!" % path)
        sys.exit(0)
    abs_path = os.path.abspath(path)
    import RedisCheck
    RedisCheck.run(abs_path)


def main():
    parser = argparse.ArgumentParser(
        description="This is a tool for detecting the security problem of spark and hadoop!")
    subparsers = parser.add_subparsers(help='commands')
    # Hadoop
    conf_check_parser = subparsers.add_parser('Hadoop', help='check the security of hadoop')
    conf_check_parser.add_argument('confFolder', action='store', help='the dir of Hadoop configuration files')
    conf_check_parser.set_defaults(func=CheckHadoop)
    # Spark
    setting_check_parser = subparsers.add_parser('Spark', help='Check the security of spark')
    setting_check_parser.add_argument('confFolder', action='store', help='the dir of Spark configuration files')
    setting_check_parser.set_defaults(func=CheckSpark)
    # MySQL
    setting_check_parser = subparsers.add_parser('MySQL', help='Check MySQL database setting')
    setting_check_parser.add_argument('username', action='store', help='the username of MySQL')
    setting_check_parser.add_argument('password', action='store', help='the password of MySQL')
    setting_check_parser.add_argument('--host', nargs='?', action='store', help='the host of MySQL, default: 127.0.0.1')
    setting_check_parser.add_argument('--port', nargs='?', action='store', help='the port of MySQL, default: 3306')
    setting_check_parser.set_defaults(func=CheckMySQL)
    # Redis
    setting_check_parser = subparsers.add_parser('Redis', help='Check the security of redis')
    setting_check_parser.add_argument('confFolder', action='store', help='the dir of redis configuration files')
    setting_check_parser.set_defaults(func=CheckRedis)
    args = parser.parse_args()
    args.func(args)


if __name__ == '__main__':
    main()
